import { IEventHandler, IEventHash, IEventsHash } from './interfaces';
/**
 * @file File defining the SequentialEvent class
 * @licence GPLv3
 * @author Gerkin
 */
/**
 * Event emitter that guarantees sequential execution of handlers. Each handler may return a **Promise**.
 *
 * @see {@link https://nodejs.org/api/events.html Node EventEmitter}.
 */
export declare class SequentialEvent {
    /**
     * Events hash attached to this SequentialEvent instance
     */
    protected __events: IEventsHash;
    /**
     * Constructs a new SequentialEvent.
     *
     * @author Gerkin
     */
    constructor();
    /**
     * Triggers each corresponding handlers in sequence.
     *
     * @param event Name of the event to trigger
     * @param args Arguments to pass to each handler. Note that those arguments will be appended with previous handler resolution value, if any.
     * @author Gerkin
     */
    emit(event: string, ...args: any[]): Promise<any>;
    /**
     * Remove one, many or all event handlers on some events.
     *
     * @param events String containing event names to unbind. You may indicate several events at once by separating with *spaces*
     * @param callback Function or array of functions to unbind on specified events
     * @author Gerkin
     */
    off(events: string, callback?: IEventHandler | IEventHandler[]): this;
    /**
     * Remove event handlers on several events at once.
     *
     * @param events Object with keys as event names containing a handler or an array of handlers to remove.
     * @author Gerkin
     */
    off(events: IEventHash): this;
    /**
     * Remove all events handlers
     *
     * @param all Pass true to delete all handlers
     * @author Gerkin
     */
    off(all: true): this;
    /**
     * Add one or many event handlers.
     *
     * @param events String containing event names to bind. You may indicate several events at once by separating with *spaces*
     * @param callback Function or array of functions to bind on specified events
     * @author Gerkin
     */
    on(events: string, callback: IEventHandler | IEventHandler[]): this;
    /**
     * Add event handlers on several events at once.
     *
     * @param events Object with keys as event names containing a handler or an array of handlers to add.
     * @author Gerkin
     */
    on(events: IEventHash): this;
    /**
     * Add one or many event handlers. These listeners will be executed once, then are removed
     *
     * @param events String containing event names to bind. You may indicate several events at once by separating with *spaces*
     * @param callback Function or array of functions to bind on specified events
     * @author Gerkin
     */
    once(events: string, callback: IEventHandler | IEventHandler[]): this;
    /**
     * Add event handlers on several events at once. These listeners will be executed once, then are removed
     *
     * @param events Object with keys as event names containing a handler or an array of handlers to add.
     * @author Gerkin
     */
    once(events: IEventHash): this;
    /**
     * Check if this instance has listeners for the provided event. Alias for `hasEvent`
     *
     * @param event Name of the event to check
     * @author Gerkin
     */
    has(event: string): boolean;
    /**
     * Check if this instance has listeners for the provided event
     *
     * @param event Name of the event to check
     * @author Gerkin
     */
    hasEvent(event: string): boolean;
}
export * from './interfaces';
