"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Handle execution of a single handler.
 *
 * @memberof SequentialEvent
 * @author Gerkin
 * @inner
 */
var emitHandler = function (handler, object, args) {
    try {
        var retVal = handler.apply(object, args);
        if ('object' === typeof retVal && 'function' === typeof retVal.then) {
            return retVal;
        }
        else {
            return Promise.resolve(retVal);
        }
    }
    catch (e) {
        return Promise.reject(e);
    }
};
/**
 * Handle execution of all handlers in sequence.
 *
 * @memberof SequentialEvent
 * @author Gerkin
 * @inner
 */
exports.emitHandlers = function (handlers, object, args) {
    handlers = exports.ensureArray(handlers);
    var promiseGen = exports.getNextPromise(handlers, object, args);
    return new Promise(promiseGen);
};
/**
 * Generate next promise for sequence.
 *
 * @memberof SequentialEvent
 * @author Gerkin
 * @inner
 */
exports.getNextPromise = function (handlers, object, args) {
    var i = 0;
    var handlersLength = handlers.length;
    return function (resolve, reject) {
        var _getNextPromise = function (prevResolve) {
            // Handle if an event handler disapeared during event dispatching
            var handlersLength2 = handlers.length;
            if (handlersLength2 !== handlersLength) {
                i -= handlersLength - handlersLength2;
                handlersLength = handlersLength2;
            }
            if (i < handlersLength) {
                var stepArgs = 'undefined' !== typeof prevResolve
                    ? args.concat([prevResolve])
                    : args.slice(0);
                var newPromise = emitHandler(handlers[i], object, stepArgs);
                newPromise.then(_getNextPromise).catch(reject);
                i++;
            }
            else {
                return resolve.call(null, prevResolve);
            }
        };
        return _getNextPromise();
    };
};
/**
 * Generate an event handler that deregister itself when executed. This handler will be executed  just once.
 *
 * @param   target    - Event emitter that will use the handler.
 * @param   eventName - Name of the event to trigger.
 * @param   eventFn   - Handler for the event.
 * @returns Function that will be executed only once.
 * @memberof SequentialEvent
 * @author Gerkin
 * @inner
 */
exports.onceify = function (target, eventName, eventFn) {
    var called = false;
    var once = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (!called) {
            target.off(eventName, once);
            called = true;
            return eventFn.apply(void 0, args);
        }
    };
    once.origFn = eventFn;
    return once;
};
var removeSingleListener = function (eventCat, callback) {
    var indexes = [
        // Check in normal events
        eventCat.indexOf(callback),
        // Check in once events
        (function () {
            var I = eventCat.length;
            for (var i = 0; i < I; i++) {
                if (eventCat[i].origFn === callback) {
                    return i;
                }
            }
            return -1;
        })(),
    ];
    var index = Math.min.apply(Math, indexes.filter(function (v) { return v >= 0; }));
    if (isFinite(index)) {
        eventCat.splice(index, 1);
        return true;
    }
    else {
        return false;
    }
};
/**
 * Remove provided `callback` from listeners of event `eventCat`.
 *
 * @memberof SequentialEvent
 * @author Gerkin
 * @inner
 */
exports.removeEventListener = function (eventCat, callback) {
    var I = callback.length;
    for (var i = 0; i < I; i++) {
        if (removeSingleListener(eventCat, callback[i])) {
            i--;
        }
    }
};
/**
 * Add an event listener to the provided event hash.
 *
 * @memberof SequentialEvent
 * @author Gerkin
 * @inner
 */
exports.addEventListener = function (eventHash, eventName, callback) {
    eventHash[eventName] = exports.ensureArray(eventHash[eventName] || []).concat(callback);
};
/**
 * Ensure that event & callback are on the associative hash format.
 *
 * @memberof SequentialEvent
 * @author Gerkin
 * @inner
 */
exports.ensureArray = function (data) {
    if ('undefined' === typeof data) {
        return [];
    }
    return (Array === data.constructor ? data : [data]);
};
exports.forEachObj = function (object, callback) {
    Object.keys(object).forEach(function (key) { return callback(object[key], key); });
};
exports.castArgsToEventsHash = function (events, callback) {
    if (typeof events === 'string' && typeof callback !== 'undefined') {
        var eventsHash_1 = {};
        var callbackArr_1 = exports.ensureArray(callback);
        events.split(' ').forEach(function (event) {
            eventsHash_1[event] = callbackArr_1;
        });
        return eventsHash_1;
    }
    else if (typeof events === 'object' && typeof callback === 'undefined') {
        var eventsHash_2 = {};
        exports.forEachObj(events, function (callback, event) { return (eventsHash_2[event] = exports.ensureArray(callback)); });
        return eventsHash_2;
    }
    else {
        throw new Error('Incorrect parameters');
    }
};
//# sourceMappingURL=utils.js.map